
; ATMega8515

; Version v0.1 27.03.2007 M'Soft Berlin

; programmierte FUSE: SUT0  - Quarz - Slowly Rising Power 

; Definitionen

; Belegungen der Atmel-Ports:

;*****************************************************************

;	PA0		0/D0	ext RAM, Flash
;	PA1		1/D1	ext RAM, Flash
;	PA2		2/D2	ext RAM, Flash
;	PA3		3/D3	ext RAM, Flash
;	PA4		4/D4	ext RAM, Flash
;	PA5		5/D5	ext RAM, Flash
;	PA6		6/D6	ext RAM, Flash
;	PA7		7/D7	ext RAM, Flash

;	PB0		T0/OC0		A14 Flash
;	PB1		T1			A15 Flash
;	PB2		AIN0		A16 Flash
;	PB3		AIN1		A17 Flash
;	PB4		SS
;	PB5		MOSI	Display DI
;	PB6		MISO
;	PB7		SCK		Display CP

;	PC0		A8		ext RAM, Flash						
;	PC1		A9		ext RAM, Flash		
;	PC2		A10		ext RAM, Flash		
;	PC3		A11		ext RAM, Flash	
;	PC4		A12		ext RAM, Flash
;	PC5		A13				 Flash
;	PC6		A14
;	PC7		A15		ext RAM	CE / Flash /CE			

;	PD0		RxD		UART In	
;	PD1		TxD		UART out (Debug)
;	PD2		INT0	Display FLM
;	PD3		INT1	Display	M
;	PD4		XCK		Display	LP
;	PD5		OC1A
;	PD6		WR		ext RAM /WR, Flash /WE
;	PD7		RD		ext RAM /OE, Flash /UE

;	PE0		INT2
;	PE1		ALE		ext RAM Latch
;	PE2		OC1B

;   externer Ram 8k ab 0x8000 - 0x9FFF (gespiegelt ab 0xA000, 0xC000, 0xE000, 32k mglich (Schaltung ndern)
;   externer Flash 16k ab 0x4000 - 0x7FFF (gespiegelt ab 0x0260 wegen AVR)

;*****************************************************************

;--- Allgemeines -----------------------------------------------

;#define USE_DEBUG

;#define	NO_FLASH			;  Flash-Erkennung umgehen, hilft, wenn man Mll in den Flash programmiert hat...

#define FLASH_PROG			; erzeugt Flashfile fr Font oder Display und Control-Daten

.equ	VERSION			= '0'
.equ	REVISION		= '8'

; benutzter Flash: 128kx8, SST 29EE010-150		0xBF, 0x07	kein Chip-Erase, Product Identification nur SST-Kommando
; benutzter Flash: 128kx8, ATMEL 29C010A-12		0x1F, 0xD5
; benutzter Flash: 128kx8, Winbond W29C020-12	0xDA, 0x45  Product Identification nur SST-Kommando

.equ	MANUFAKTURER	= 0xDA
.equ	DEVICE_ID		= 0x45

.equ    F_CPU           = 16000000

.equ    CYCLES_PER_US   = ((F_CPU+500000)/1000000)

.equ	IRQ_TIME		= 5						; IRQ-Zeit in !!! s !!!

.equ	F_REFRESH		= 200000				; Refresh in Hz

.equ	VORTEILER		= 1
.equ	COUNT_IN		= F_CPU/VORTEILER
.equ    TCONST_TIME     = (F_CPU/F_REFRESH)-1		; 200kHz Refresh-IRQ 								

.equ    UART_BAUD_RATE  = 38400
.equ    UART_BAUD_SELECT= (F_CPU/(UART_BAUD_RATE*16))-1

.equ	TIMEOUT			= 10					; XOFF-Timeout-Zeit in ca. 50ms Schritten								

; Interne Konstanten

.equ	LINE_LEN		= 480
.equ	X_SIZE			= LINE_LEN/8
.equ	Y_SIZE			= 64

.equ	H_TABULATOR		= 0						; in Pixel oder in Fontbreite oder ???
.equ	V_TABULATOR		= 0						; in Pixel oder in Fontbreite oder ???

.equ	SCREEN_ANZ		= 2						; Anzahl Bildspeicher im Ram

.equ	BANK_ANZ		= 8						; Anzahl Flash-Bnke je 16k

.equ	DEF_FONT_H		= 8						; Hhe des Rom-Fonts
.equ	DEF_FONT_W		= 8						; Breite des Rom-Fonts
.equ	DEF_FONT_B		= 1						; Abtahl Bytes pro Zeile des Rom-Fonts

.equ	X_ON			= 0x11
.equ	X_OFF			= 0x13
.equ	ESCAPE			= 0x1B

;**************************** Ram-Reservierungen ******************

;--- Internes Ram -------------------------------------------------

.DSEG

.ORG	$060

SCREEN:		.byte	 2							; L/H aktueller Displayram zum Beschreiben 
DISPLAY:	.byte	 2							; L/H aktuell agezeigter Displayram fr Display-Refresh

FONT_START:	.byte	 2							; L/H Startadresse des aktuellen Fonts
FONT_BANK:	.byte	 1							; Bank im externen Flash
FONT_HEIGH:	.byte	 1							; Hhe in Pixel
FONT_WIDTH:	.byte	 1							; Breite in Pixel
FONT_BYTE:	.byte	 1							; Breite in Byte

FLASH_BUF:	.byte	 2							; letze gelesene Adresse+1
FLASH_ADR:	.byte	 2							; Flash-Adresse beim Programmieren
FLASH_LEN:	.byte	 1							; Anzahl zu programmierende Pages
FLASH_BANK:	.byte	 1							; Flashbank, die programmiert werden soll

.ORG	$E000									; !!! bei nderungen mu Screen-Map neu geschrieben werden !!!
	; externer Ram, bei 32k ab 0x8000,
	; bei 8k gespiegelt an 0xA000 und 0xE000, 0x8000-0x9FFF und 0xC000-0xBFFF sind frei verfgbat
	; A13 ist dort CE des 8k-Ram und dieser nicht selektiert

DDRAM_0:	.byte	X_SIZE*Y_SIZE				; DDRAM 0
DDRAM_1:	.byte	4096						; >(X_SIZE*Y_SIZE) DDRAM 1 auch 4k-Buffer zum Flash programmieren !!

FLASH_END:	.byte	 128						; XOFF bei 0x0x9F00, 128 Byte Reserve bis LINE_OFFSET 

LINE_OFFSET:.byte	2*64						; Tabelle Offset der Zeilenanfnge
													
;****************** Port-Zuordnung und Register-Bits ******************

;--- LCD ----------------------------------------------------------

.equ	LCD_DATA	= PB5						; 1 - Datenbit		- SPI MOSI
.equ	LCD_CLOCK	= PB7						; 5 - Pixelclock	- SI CLOCK

.equ	LCD_PORT	= PORTD
.equ	LCD_PIN		= PIND

.equ	LCD_FLM		= PD2						; 2 - First Line Marker (VSync)
.equ	LCD_M		= PD3						; 3 - AC-Drive
.equ	LCD_LP		= PD4						; 4 - Latch-Puls (HSync)

.equ	M_MASK		= (1<<LCD_M)

;--- externes Flash -------------------------------------------------

.equ	FLASH_BUFFER	= DDRAM_1				; 4k Buffer zum Flash programmieren

.equ	FLASH_START		= 0x4000

.equ	SYSTEM_MAP		= 0x0000				; je 128 Byte - 1 Page im Flash
.equ	DISPLAY_MAP		= 0x0080
.equ	FONT_MAP		= 0x0100

.equ	FONT_ANZ		= FLASH_START+FONT_MAP+1	; Daten im externen Flash
.equ	FONT_LISTE		= FLASH_START+FONT_MAP+2	; Start Fontliste im Flash

.equ	DISPLAY_ANZ		= FLASH_START+DISPLAY_MAP+1	; Daten im externen Flash
.equ	DISPLAY_LISTE	= FLASH_START+DISPLAY_MAP+2	; Start Displayliste im Flash

.equ	BANK_PORT		= PORTB

.equ	BANK_0			= 0b00000000
.equ	BANK_1			= 0b00000001
.equ	BANK_2			= 0b00000010
.equ	BANK_3			= 0b00000011
.equ	BANK_4			= 0b00000100
.equ	BANK_5			= 0b00000101
.equ	BANK_6			= 0b00000110
.equ	BANK_7			= 0b00000111

; bei 256kx8 Flash
.equ	BANK_8			= 0b00001000
.equ	BANK_9			= 0b00001001
.equ	BANK_10			= 0b00001010
.equ	BANK_11			= 0b00001011
.equ	BANK_12			= 0b00001100
.equ	BANK_13			= 0b00001101
.equ	BANK_14			= 0b00001110
.equ	BANK_15			= 0b00001111

;--- Flags --------------------------------------------------------

.equ	FLAG_FLASH		= 0						; Listen sind im internen Flash!

; Flasg fr flash_programm

.equ	FLAG_ADR		= 1						; zu programmierende Startadresse Flash gespeichert
.equ	FLAG_CONTROL	= 2						; wird bei Control-Map gesetzt, damit immer Bank 0 programmiert wird

;--- lower Register -----------------------------------------------

.def	MUL_LOW		= r0
.def	MUL_HIGH	= r1

.def	ZERO		= r2						; 0 fr Berechnungen

.def	FONT_NUM	= r3

.def	REGSAVE		= r13						; Sicherung fr SREG in Refresh-IRQ
.def	TEMP_IRQ	= r14						; temporres Register fr Refresh-IRQ
.def	M_MASK_REG	= r15						; enthlt Maske fr M in Refresh-IRQ

;--- higher Register ----------------------------------------------

.def	X_REG		= r16						; Register fr Refresh-IRQ
.def	Y_REG		= r17

.def	X_POS		= r18						; aktuelles Byte X
.def	Y_POS		= r19						; aktuelle Zeile 
.def	BIT_POS		= r20						; ajtuelle Bit in X_POS

.def	FLAGS		= r21						; allgemeines Flagregister

.def    TEMP_0      = r22						; 4 temporre  8Bit-Register
.def    TEMP_1      = r23
.def    TEMP_2      = r24
.def    TEMP_3      = r25

	; X-Register zeigt auf DDRAM fr Displayrefresh
	; Y-Register enthlt Cursor-Position 0

; r26/r27 - X, r28/r29 - Y, r30/r31 - Z

;--- Macros -------------------------------------------------------

.macro  load_p                  		    	; ldt 16Bit @1 nach @0 High und @0 Low
        ldi     @0H,HIGH(@1)
        ldi     @0L,LOW(@1)
.endmacro

.macro  setbank                  		    	; ldt 16Bit @1 nach @0 High und @0 Low
        push	TEMP_0
		ldi     TEMP_0,@0
        out     BANK_PORT,TEMP_0
		pop		TEMP_0
.endmacro

