
.include 	"m8515def.inc"

.include	"Definitionen.inc"

;****************************** Verktor-Liste ****************************

.CSEG
                rjmp    Reset
.org OC0addr
				rjmp	irq_refresh

;****************************** Programm-Beginn **********************

;*********************************************************************
;  Initialisierungsroutine
;*********************************************************************

.ORG	INT_VECTORS_SIZE

; Hardware initialisieren

Reset:  ldi     TEMP_0,low(RAMEND)          	; Stack an das interne Ram-Ende
        out     SPL,TEMP_0
        ldi     TEMP_0,high(RAMEND)
        out     SPH,TEMP_0

; Externer Ram ein ***************************************************

		ldi		TEMP_0,(1<<SRE)					; ext. Ram/Flash ein, 64k, 0 Wait
		out		MCUCR,TEMP_0

; Port A *************************************************************

        ser     TEMP_0							; Alle PullUp ein bzw. Ausgang auf High
        out     PORTA,TEMP_0
        out     DDRA,TEMP_0

; Port B *************************************************************

        ser		TEMP_0							; Alle PullUp ein bzw. Ausgang auf High
		out     PORTB,TEMP_0
        out     DDRB,TEMP_0	

; Port C *************************************************************

        ser     TEMP_0							; Alle PullUp ein bzw. Ausgang auf High
        out     PORTC,TEMP_0
        out     DDRC,TEMP_0	

; Port D *************************************************************

		ser		TEMP_0							; Alle PullUp ein bzw. Ausgang auf High
        out     PORTD,TEMP_0
		out     DDRD,TEMP_0

; Port E *************************************************************

		ser		TEMP_0							; Alle PullUp ein bzw. Ausgang auf High
        out     PORTE,TEMP_0
        out     DDRE,TEMP_0

; TIMER 8Bit *********************************************************

		ldi		TEMP_0,TCONST_TIME
		out		OCR0,TEMP_0

		ldi		TEMP_0,(1<<WGM01)|(1<<CS00)		; CTC, Vorteiler = 1
		out		TCCR0,TEMP_0

		ldi 	TEMP_0,(1<<OCIE0)				; Interrupt Display Refresh
		out 	TIMSK, TEMP_0

; UART ***************************************************************

        ldi     TEMP_0,high(UART_BAUD_SELECT)
        out     UBRRH,TEMP_0
        ldi     TEMP_0,low(UART_BAUD_SELECT)
        out     UBRRL,TEMP_0
        ldi     TEMP_0,(1<<TXEN) | (1<<RXEN)
        out     UCSRB,TEMP_0
        ldi     TEMP_0,(1<<URSEL)+(1<<UCSZ1)+(1<<UCSZ0)
        out     UCSRC,TEMP_0

; SPI ****************************************************************

        ldi     TEMP_0,(1<<MSTR) | (1<<SPE)	| (1<<DORD)	; Enable SPI-Master, Clock 4MHz bei 16MHz Clock
        out     SPCR,TEMP_0	    	        	; SPI initialisieren
        in      TEMP_0,SPSR   	            	; Flags lschen

;*********************************************************************				 

; Software initialisieren

		clr		ZERO
		clr		FLAGS

		rcall	uart_send_newline
		load_p	Z,(2*text_reset)
		rcall	uart_send_text
		rcall	uart_send_newline

		rcall	flash_init						; Test, ob Flash ok, sonst internen Font usw. nutzen
		rcall	display_init					; Init Register, Font Default, Display Clear DDRAM 0 und 1

		sei
		
										 
;*********************************************************************
;  Main-Loop
;*********************************************************************

main:

#ifdef	USE_DEBUG
/*
		rcall	debug_flash_system
		rcall	uart_send_newline

		rcall	debug_flash_screen
		rcall	uart_send_newline

		rcall	debug_flash_font
		rcall	uart_send_newline

		rcall	debug_font_daten
		rcall	uart_send_newline

		rcall	debug_flash_daten
		rcall	uart_send_newline
*/
#endif

main_loop:
		rcall	uart_receive_byte
		cpi		TEMP_0,' '
		brlo	main_execute					; Kommando

		push	TEMP_0
		rcall	display_write_char				; Zeichen ausgeben
		pop		TEMP_0
		rcall	uart_send_byte

		rjmp	main_loop

main_execute:

		load_p	Z,(2*kommando_liste)

		cpi		TEMP_0,ESCAPE
		brne	main_execute_command			; normales Kommando

		load_p	Z,(2*esc_kommando_liste)		; ESC Kommando 

main_esc_loop:
		rcall	uart_receive_byte
		cpi		TEMP_0,'a'
		brlo	main_loop						; Fehler, kein Kleinbuchstabe

		cpi		TEMP_0,'z'+1
		brsh	main_loop						; Fehler, Sonderzeichen

		subi	TEMP_0,'a'						; Start ESC-Sequenz a...z

main_execute_command:
		lsl		TEMP_0							; * 2 wegen Word
		add		ZL,TEMP_0						; Offset dazu
		adc		ZH,ZERO
 
		lpm		TEMP_0,Z+						; Adresse holen
		lpm		TEMP_1,Z

		movw	ZL,TEMP_0						; nach Z

		icall									; Funktion aufrufen

		rjmp	main_loop

;*********************************************************************
;  Ende Main-Loop
;*********************************************************************


;***************************************************************************
;  Programm Ende 
;***************************************************************************

#ifdef USE_DEBUG

.include	"debug.inc"

#endif

.include	"subroutinen.inc"
.include	"display.inc"
.include	"flash.inc"
.include	"irq.inc"
.include	"uart.inc"
.include	"tabellen.inc"

#ifdef	FLASH_PROG

.include	"control_map.inc"

#endif


