;************************** Display Routinen *************************

;*********************************************************************
;  Clear Display lscht den aktuellen Displayram (NewPage - 0x0C)
;  
;.
;  Scratch-Reg: -
;*********************************************************************

display_clear:
		push	TEMP_0
		push	YH
		push	YL
		push	ZH
		push	ZL

		lds		YH,SCREEN+1						; Startadresse
		lds		YL,SCREEN

		load_p	Z, (X_SIZE*Y_SIZE)				; Displaygre in Byte

		clr		TEMP_0

display_clear_loop:
		st		Y+,TEMP_0
		sbiw	Z,1
		brne	display_clear_loop

		pop		ZL
		pop		ZH
		pop		YL
		pop		YH
		pop		TEMP_0

;*********************************************************************
;  Display Home setzt Cursor auf 0,0,0
;  
;.
;  Scratch-Reg: Z, TEMP_0
;*********************************************************************

display_home:
		clr		X_POS								; Cursor auf Anfang
		clr		Y_POS
		clr		BIT_POS

		ret

;*********************************************************************
;  Display Newline erzeugt Return (CR- 0x0D) und Linefeed (LF - 0x0A)
;  
;
;  Scratch-Reg: -
;*********************************************************************

display_newline:
		clr		X_POS
		clr		BIT_POS
		
		; weiter mit display_linefeed

;*********************************************************************
;  Display Linefeed erzeugt Linefeed (LF - 0x0A)
;  
;.
;  Scratch-Reg: -
;*********************************************************************

display_linefeed:
		push	TEMP_0

		lds		TEMP_0,FONT_HEIGH
		add		Y_POS,TEMP_0
		cpi		Y_POS,Y_SIZE
		brlo	display_linefeed_end

		clr		Y_POS

display_linefeed_end:
		pop		TEMP_0

		ret

;*********************************************************************
;  Display Return erzeugt Return (CR- 0x0D)
;  
;.
;  Scratch-Reg: -
;*********************************************************************

display_return:
		clr		X_POS
		clr		BIT_POS

		ret

;*********************************************************************
;  Display Backspace
;  
;.
;  Scratch-Reg: -
;*********************************************************************

display_backspace:

		ret

;*********************************************************************
;  Display H-Tabulator
;  
;.
;  Scratch-Reg: -
;*********************************************************************

display_h_tab:
		push	TEMP_0

		ldi		TEMP_0,H_TABULATOR

display_h_tab_end:
		pop		TEMP_0

		ret

;*********************************************************************
;  Display V-Tabulator
;  
;.
;  Scratch-Reg: -
;*********************************************************************

display_v_tab:
		push	TEMP_0

		ldi		TEMP_0,V_TABULATOR

display_v_tab_end:
		pop		TEMP_0

		ret

;*********************************************************************
;  Display Init initialisiert alle Displaywerte
;  In: .
;.
;  Scratch-Reg: Y, Z, TEMP_0, 
;*********************************************************************

display_init:
		clr		YH
		clr		YL
		load_p	Z,LINE_OFFSET					; Tabelle der Zeilenanfnge aufbauen
		ldi		TEMP_0,Y_SIZE					; Anzahl Zeilen
								
display_init_loop:		
		st		Z+,YH
		st		Z+,YL
		adiw	Y,X_SIZE						; Zeilenlnge dazu		
		dec		TEMP_0
		brne	display_init_loop

		ldi		X_REG,X_SIZE+1					; der 61. Zyklus ist Latch/Zeilenwechsel
		ldi		Y_REG,Y_SIZE					; Anzahl Zeilen
		ldi		TEMP_0,M_MASK					; fr EOR des M-Signals
		mov		M_MASK_REG,TEMP_0

		sbrs	FLAGS,FLAG_FLASH				; gltiger Flash vorhanden?
		ldi		TEMP_0,0						; nein, ohne Flash nur Scrren 0

		sbrc	FLAGS,FLAG_FLASH				; gltiger Flash vorhanden?
		ldi		TEMP_0,SCREEN_ANZ-1				; ja, Anzahl Screens, die beschreibbar sind

display_init_bank_loop:
		push	TEMP_0
		rcall	display_set_screen_init			; setzt Screen
		rcall	display_clear
		pop		TEMP_0
		dec		TEMP_0							; nchste Bank
		brpl	display_init_bank_loop			; weiter bis 0xFF, Screen 0 bleibt dann ausgewhlt

		ldi		TEMP_0,0
		rcall	display_set_display_init		; setzt Anzeige auf Display 0

		lds		XH,DISPLAY+1					; Refresh-Adresse setzen
		lds		XL,DISPLAY

		rcall	display_home					; Cursor Home

		load_p	Z,(2*text_start_0)				; Starttext ausgeben
		rcall	show_text_line
					
		load_p	Z,(2*text_start_1)
		rcall	show_text_line

		sbrc	FLAGS,FLAG_FLASH
		rjmp	display_init_end

		load_p	Z,(2*text_no_flash_0)			; Text zustzlich bei kein Flash erkannt
		rcall	show_text_line

		load_p	Z,(2*text_no_flash_1)
		rcall	show_text_line

display_init_end:
		ret

;*********************************************************************
;  Set Screen Setzt Screen zum Beschreiben
;  In: TEMP_0 Screennummer
;.
;  Scratch-Reg: Z, TEMP_0
;*********************************************************************

display_set_screen:
		sbrs	FLAGS,FLAG_FLASH				; Flash da?
		rjmp	display_set_screen_end			; nein

		rcall	uart_receive_byte
		subi	TEMP_0,'0'

		cpi		TEMP_0,SCREEN_ANZ
		brsh	display_set_screen_end			; Fehler

display_set_screen_init:
		lsl		TEMP_0
		load_p	Z,(2*screen_liste)				; Listenadresse holen
		add		ZL,TEMP_0
		adc		ZH,ZERO

		lpm		TEMP_0,Z+
		sts		SCREEN,TEMP_0
		lpm		TEMP_0,Z
		sts		SCREEN+1,TEMP_0
		
		clr		FONT_NUM						; Font 0 setzen
		mov		TEMP_0,FONT_NUM
		rcall	display_set_font_init

display_set_screen_end:

		ret

;*********************************************************************
;  Show DDRAM setzt Bank 0 des Displayrams zum Anzeigen
;  In:			- .
;. Out:			-
;  Scratch-Reg:	Z, TEMP_0, TEMP_1
;*********************************************************************

display_set_display:
		sbrs	FLAGS,FLAG_FLASH				; Flash da?
		rjmp	display_set_display_end			; nein

		setbank	BANK_0
		lds		TEMP_1,DISPLAY_ANZ

		rcall	uart_receive_byte
		subi	TEMP_0,'0'

		cp		TEMP_0,TEMP_1
		brsh	display_set_display_end			; Fehler
	
display_set_display_init:
		sbrs	FLAGS,FLAG_FLASH				; Flash erkannt?
		rjmp	display_set_display_init_flash

		lsl		TEMP_0							; *2, Word-Tabelle
		load_p	Z,DISPLAY_LISTE
		add		ZL,TEMP_0
		adc		ZH,ZERO

		ld		TEMP_0,Z+						; L Byte
		sts		DISPLAY,TEMP_0
		ld		TEMP_0,Z						; H Byte
		sts		DISPLAY+1,TEMP_0
		
		rjmp	display_set_display_end

display_set_display_init_flash:
		load_p	Z,DDRAM_0						; nur Display 0 verfgbar
		sts		DISPLAY,ZL
		sts		DISPLAY+1,ZH

display_set_display_end:
		ret

;*********************************************************************
;  Set Font setzt die Daten eines Fonts
;  In:  TEMP_0 Fontnummer
;.
;  Scratch-Reg: TEMP_0, TEMP_1
;*********************************************************************

display_set_font:
		sbrs	FLAGS,FLAG_FLASH				; Flash da?
		rjmp	display_set_font_end

		setbank	BANK_0
		lds		TEMP_1,FONT_ANZ

		rcall	uart_receive_byte
		subi	TEMP_0,'0'

		cp		TEMP_0,TEMP_1
		brsh	display_set_font_end

display_set_font_init:
		push	ZL
		push	ZH
		push	TEMP_1
		
		mov		FONT_NUM,TEMP_0					; aktuelle Fontnummer setzen

		setbank	BANK_0
		load_p	Z,FONT_LISTE					; Zeiger auf Fontliste im Flash	

		lsl		TEMP_0							; *2
		mov		TEMP_1,TEMP_0
		lsl		TEMP_0							; *4
		add		TEMP_0,TEMP_1					; *2 + *4 = *6

		add		ZL,TEMP_0
		adc		ZH,ZERO

		ld		TEMP_0,Z+
		sts		FONT_START,TEMP_0
		ld		TEMP_0,Z+
		sts		FONT_START+1,TEMP_0
		ld		TEMP_0,Z+
		sts		FONT_WIDTH,TEMP_0
		ld		TEMP_0,Z+
		sts		FONT_HEIGH,TEMP_0
		ld		TEMP_0,Z+
		sts		FONT_BYTE,TEMP_0
		ld		TEMP_0,Z+
		sts		FONT_BANK,TEMP_0

		pop		TEMP_1
		pop		ZH
		pop		ZL

display_set_font_end:
		ret

;******************************* Sub-Debug *********************************
; Sub: Sendet Text aus Programm-Flash an das Display
; Parameter:	Z enthlt den Zeiger auf den Text im ROM
; Return:		-
; Scratch-Reg:	-
;***************************************************************************

display_write_text:
		push	ZH
		push	ZL
		push	TEMP_0

display_write_text_loop:
        lpm		TEMP_0,Z+
		tst		TEMP_0
		breq	display_write_text_end				; Ende

        rcall   display_write_char

		rjmp    display_write_text_loop				; weiter

display_write_text_end:
		pop		TEMP_0
 		pop		ZL
		pop		ZH

        ret

;*********************************************************************
;  Write Char schreibt ein Zeichen an Cursor-Position
;  In:  TEMP_0 ASCII fr Ausgabe
;.
;  Scratch-Reg: Z, TEMP_0, TEMP_1, TEMP_2, TEMP_3
;*********************************************************************

display_write_char:
		sbrs	FLAGS,FLAG_FLASH				; nur mit Flash Zeichen ausgeben!
		rjmp	display_write_char_ignore

		push	ZH
		push	ZL
		push	TEMP_1
		push	TEMP_2
		push	TEMP_3

		load_p	Z,LINE_OFFSET					; Tabelle Zeilenanfnge
		mov		TEMP_1,Y_POS					; Position holen
		lsl		TEMP_1							; *2
		add		ZL,TEMP_1
		adc		ZH,ZERO
		ld		YH,Z+							
		ld		YL,Z
		add		YL,X_POS
		adc		YH,ZERO							; Y zeigt jetzt auf aktuelles Byte
				
		lds		TEMP_1,SCREEN
		add		YL,TEMP_1						; Startadresse dazu
		lds		TEMP_1,SCREEN+1
		adc		YH,TEMP_1	

		lds		ZH,FONT_START+1					; Adresse holen
		lds		ZL,FONT_START
		lds		TEMP_2,FONT_BANK				; Bank holen
		out		BANK_PORT,TEMP_2				; und setzen

		subi	TEMP_0,0x20

		lds		TEMP_2,FONT_HEIGH				; Hhe fr Zeilen-Zhler

		lds		TEMP_3,FONT_BYTE				; Breite in Byte holen

		mul		TEMP_3,TEMP_2					; Zeichengre Anzahl Bytes Breite * Hhe
		mov		TEMP_3,MUL_LOW					; Zeichengre in Byte holen (maximal 256 Byte, 32x48 -> 192 Byte
		
		mul		TEMP_0,TEMP_3					; ASCII * Anzahl pro Byte
		add		ZL,MUL_LOW						; Zum Tabellenanfang dazu
		adc		ZH,MUL_HIGH						; Z zeigt auf erstes Byte des Zeichens		
		
display_write_char_loop:
		lds		TEMP_3,FONT_BYTE				; Zhler fr Bytes pro Zeile

display_write_char_byte_loop:
		sbrc	FLAGS,FLAG_FLASH				; Flash vorhanden?
		ld		TEMP_0,Z+						; Char-Byte holen
		sbrs	FLAGS,FLAG_FLASH
		lpm		TEMP_0,Z+						; kein Flash, also aus Rom

		tst		BIT_POS							; bei 0 komplettes Byte
		breq	display_write_char_byte_loop_ok
/*
		push	TEMP_0							; merken fr die restlichen Bits im nchsten Byte
		push	TEMP_2							; fr Bit-Zhler
		push	TEMP_3							; fr bearbeitet Pixel von Font-Breite
		lds		TEMP_3,FONT_WIDTH				; Font-Breite in Pixel
		mov		TEMP_2,BIT_POS					; Bit-Position als Zhler
		clr		BIT_POS							; fr neue Bit-Position							
		clr		TEMP_1

display_write_char_byte_loop_bit:
		lsr		TEMP_0
		ror		TEMP_1		
		inc		BIT_POS							; Bitposition im neuen Byte
		dec		TEMP_3							; bearbeitete Fontbreite
		dec		TEMP_2
		brne	display_write_char_byte_loop_bit
*/
display_write_char_byte_loop_ok:		
		st		Y+,TEMP_0						; Char-Byte ins Displayram

		dec		TEMP_3							; Alle Bytes einer Zeile?
		brne	display_write_char_byte_loop	; nein					

		ldi 	TEMP_0,X_SIZE					; Zeilenlnge dazu
		lds		TEMP_3,FONT_BYTE				; Anzahl Bytes pro Zeile
		sub		TEMP_0,TEMP_3

		add		YL,TEMP_0
		adc		YH,ZERO							; eine Zeile weiter

		dec		TEMP_2							; Zhler Zeichen-Zeilen 
		brne	display_write_char_loop

		add		X_POS,TEMP_3					; Zeichensatzbreite in Byte dazu

		cpi		X_POS,X_SIZE					; Zeilen-Ende?
		brlo	display_write_char_end

		rcall	display_newline
				
		cpi		Y_POS,Y_SIZE					; letzte Zeile?
		brlo	display_write_char_end			; nein

		clr		Y_POS							; zurck auf Anfang

display_write_char_end:
		pop		TEMP_3
		pop		TEMP_2
		pop		TEMP_1
		pop		ZL
		pop		ZH

display_write_char_ignore:
		ret


