;************************** Flash Routinen *************************

;******************************* Sub-Flash *********************************
; Sub: Sendet Control-Kommando an den Flash
; Parameter:    TEMP_0 Control-Byte
; Return:       -
; Scratch-Reg:  -
;***************************************************************************

flash_control:
		push	ZH
		push	ZL
		push	TEMP_1
		push	TEMP_2
		
		ldi		TEMP_1,0xAA
		load_p	Z,0x5555						; Kommand 1				0101 0101
		setbank	BANK_1
		st		Z,TEMP_1						; und schreiben

		ldi		TEMP_1,0x55
		load_p	Z,0x2AAA | 0x4000				; Kommand 2				0010 1010
		setbank	BANK_0
		st		Z,TEMP_1						; und schreiben

		load_p	Z,0x5555						; Kommand 1				0101 0101
		setbank	BANK_1
		st		Z,TEMP_0						; und schreiben

		pop		TEMP_2
		pop		TEMP_1
		pop		ZL
		pop		ZH

		ret

;******************************* Sub-Flash *********************************
; Sub: Sendet Flash-Manufakturer/Device-ID ber UART
; Parameter:    -
; Return:       TEMP_0 Manufakturer, TEMP_1 DeviceID
; Scratch-Reg:  -
;***************************************************************************

flash_get_id:

.if (MANUFAKTURER == 0xBF) | (MANUFAKTURER == 0xDA)

		ldi		TEMP_0,0x80						; Manufakturer/Device lesen, alte SST-Version!
		rcall	flash_control

		ldi		TEMP_0,0x60
		rcall	flash_control

		rcall	wait_10us

.else

		ldi		TEMP_0,0x90						; Manufakturer/Device lesen, alte SST-Version!
		rcall	flash_control

		rcall	wait_20ms

.endif

		setbank	BANK_0

		lds		TEMP_0,FLASH_START				; Manufakturer 0xBF aus 0x0000

		push	TEMP_0

		lds		TEMP_1,FLASH_START+1			; Device 0x07		aus 0x0001	

		ldi		TEMP_0,0xF0						; wieder ausschalten
		rcall	flash_control

		pop		TEMP_0

		ret

;******************************* Sub-Flash *********************************
; Sub: Testet, ob programmierter Flash vorhanden ist (System-Map)
; Parameter:    
; Return:       TEMP_0 Manufakturer, TEMP_1 Chiptyp, TEMP_2 Version, TEMP_3 Revision
; Scratch-Reg:  -
;***************************************************************************

flash_check:
		rcall	flash_get_id

		setbank	BANK_0

		lds		TEMP_2,FLASH_START+2			; Manufakturer lesen
		lds		TEMP_3,FLASH_START+3			; DeviceID lesen

		cp		TEMP_0,TEMP_2					; Manufakturer ok?
		brne	flash_check_end					; nein

		cp		TEMP_1,TEMP_3					; ok?
		brne	flash_check_end					; nein
		
		lds		TEMP_2,FLASH_START				; Kennung lesen
		lds		TEMP_3,FLASH_START+1
		or		TEMP_2,TEMP_3
		brne	flash_check_end					; Z = 0, kein Flash oder nicht programmiert

		lds		TEMP_2,FLASH_START+4			; Version lesen
		lds		TEMP_3,FLASH_START+5			; Revision lesen

flash_check_end:
		
		ret

;******************************* Sub-Flash *********************************
; Sub: Init Flash System-Map, Font-Map und Screen-Map auswerten
; Parameter:    
; Return:       -
; Scratch-Reg:  -
;***************************************************************************

flash_init:
		rcall	flash_check

#ifdef	NO_FLASH

		clz										; Z = 0, kein Flash erkannt setzen

#endif
		push	TEMP_1							; DeviceID
		push	TEMP_0							; Manufakturer		
		brne	flash_init_flash_no

		load_p	Z,(2*text_flash_ok)
		rcall	uart_send_text

		sbr		FLAGS,(1<<FLAG_FLASH)			; Flash vorhanden

flash_init_flash_no:	
		load_p	Z,(2*text_flashtyp)
		rcall	uart_send_text
		pop		TEMP_0
		rcall	uart_send_byte_hex
		ldi		TEMP_0,' '
		rcall	uart_send_byte
		pop		TEMP_0
		rcall	uart_send_byte_hex
		rcall	uart_send_newline
		rcall	uart_send_newline

flash_init_flash_end:
		ret

;******************************* Sub-Flash *********************************
; Sub: Schreibt max. 16k + Control-Map in den Flash
; Parameter:    -
; Return:       -
; Scratch-Reg:  -
;***************************************************************************

flash_write_daten:
		rcall	uart_send_newline

		lds		TEMP_0,FLASH_BUF			; L-Byte
		lds		TEMP_1,FLASH_BUF+1			; H-Byte
		subi	TEMP_0,low(DDRAM_1)			; zu programmierende Flash-Lnge
		sbci	TEMP_1,high(DDRAM_1)
		lsl		TEMP_1						; Anzahl Blcke berechnen
		brne	flash_write_daten_ok		; mindestens 1 Block

		inc		TEMP_1						; weniger als 1 Block, also 1 Block schreiben

flash_write_daten_ok:
		cbr		FLAGS,(1<<FLAG_CONTROL)		; Flag fr Control-Map erstmal lschen
		load_p	Z,DDRAM_1
		lds		YH,FLASH_ADR+1
		lds		YL,FLASH_ADR
		cpi		YH,0x40						; Daten?
		brsh	flash_write_daten_loop		; ja

		ori		YH,0x40						; Flashadresse Control-Map korrigieren
		sbr		FLAGS,(1<<FLAG_CONTROL)		; Flag setzen fr Bankumschaltung

		; Z Ramadresse, Y Flashadresse, TEMP_1 Anzahl Pages

flash_write_daten_loop:

		ldi		TEMP_0,0x80						; Data Protect Disable
		rcall	flash_control

		ldi		TEMP_0,0x20
		rcall	flash_control

		rcall	wait_20ms
		rcall	wait_20ms

		ldi		TEMP_0,0xA0						; Page Write
		rcall	flash_control

		sbrs	FLAGS,FLAG_CONTROL				; Controlmap?
		lds		TEMP_0,FLASH_BANK				; nein, Bank holen
		sbrc	FLAGS,FLAG_CONTROL
		ldi		TEMP_0,BANK_0					; ja, immer Bnak 0 programmieren!
		out		BANK_PORT,TEMP_0				; und setzen
		
		ldi		TEMP_2,128						; Page Lnge

flash_write_daten_block_loop:
		ld		TEMP_0,Z+						; Daten holen
		st		Y+,TEMP_0						; und schreiben
		dec		TEMP_2
		brne	flash_write_daten_block_loop

		rcall	wait_20ms
		rcall	wait_20ms
		rcall	wait_20ms

		ldi		TEMP_0,'#'
		rcall	uart_send_byte

		dec		TEMP_1
		brne	flash_write_daten_loop

		load_p	Z,FLASH_BUFFER					; nach dem Schreiben Buffer auf Anfang
		rcall	uart_send_newline

		ret

;******************************* Sub-Flash *********************************
; Sub: Flash Receive Init setzt Screen/Display, holt Bank und gibt Texte aus
; Parameter:    -
; Return:       -
; Scratch-Reg:  -
;***************************************************************************

flash_programm_init:
		clt
									
		ldi		TEMP_0,0
		rcall	display_set_screen_init
		rcall	display_clear
		ldi		TEMP_0,0
		rcall	display_set_display_init

		load_p	Z,(2*text_flashprog)
		rcall	display_write_text
		rcall	uart_send_newline
		rcall	uart_send_text

		rcall	uart_receive_byte				; Bank holen
		subi	TEMP_0,'0'

		cpi		TEMP_0,BANK_ANZ
		brlo	flash_programm_init_bank_ok

		rcall	display_newline
		rcall	uart_send_newline
		load_p	Z,(2*text_flasherr_1)
		rcall	display_write_text
		rcall	uart_send_text
		rcall	uart_send_newline
		
		set										; T = 1, Fehler
		rjmp	flash_programm_init_end

flash_programm_init_bank_ok:
		sts		FLASH_BANK,TEMP_0				; Bank merken

		ori		TEMP_0,'0'						; nach ASCII

		push	TEMP_0
		rcall	display_write_char
		pop		TEMP_0
		rcall	uart_send_byte

		rcall	display_newline
		rcall	uart_send_newline
		rcall	display_newline
		rcall	uart_send_newline

		load_p	Z,(2*text_flashfile)
		rcall	display_write_text
		rcall	uart_send_text
		
		rcall	display_newline
		rcall	uart_send_newline

		load_p	Z,FLASH_BUFFER					; Z Adresse auf Buffer

flash_programm_init_end:
		ret

;******************************* Sub-Flash *********************************
; Sub: Empfngt max 16k Daten und programmiert Flash-Bank
; Parameter:    -
; Return:       -
; Scratch-Reg:  -
;***************************************************************************

flash_programm:
		rcall	flash_programm_init
		brtc	flash_programm_line_loop	

		rjmp	flash_programm_end				;  Sprungweite zu gro fr direkten brts nach flash_receive_end
					
flash_programm_line_loop:						; zeilenweise lesen
		rcall	uart_receive_byte				; Byte holen
		
		cpi		TEMP_0,':'						; Kennung?
		brne	flash_programm_line_loop			; nein, weiter warten

		rcall	uart_receive_byte_hex			; Anzahl holen
		brtc	flash_programm_line_loop_0		; wenn T = 0 ok, weiter
		
		rjmp	flash_programm_end				; Fehler, Sprungweite zu gro fr direkten brts nach flash_receive_end

flash_programm_line_loop_0:
		mov		TEMP_2,TEMP_0					; als Start Prfsumme
		mov		TEMP_3,TEMP_0					; Bytezhler in TEMP_3

		rcall	uart_receive_byte_hex			; H-Byte Adresse holen
		brtc	flash_programm_line_loop_1		; wenn T = 0 ok, weiter
		
		rjmp	flash_programm_end				; Fehler, Sprungweite zu gro fr direkten brts nach flash_receive_end
		
flash_programm_line_loop_1:
		mov		YH,TEMP_0						; Adresse H-Byte nach Y
		add		TEMP_2,TEMP_0					; und zu Prfsumme dazu
		
		rcall	uart_receive_byte_hex			; L-Byte Adresse holen
		brtc	flash_programm_line_loop_2		; wenn T = 0 ok, weiter
		
		rjmp	flash_programm_end				; Fehler, Sprungweite zu gro fr direkten brts nach flash_receive_end
		
flash_programm_line_loop_2:
		mov		YL,TEMP_0						; Adresse L-Byte nach Y
		add		TEMP_2,TEMP_0					; und zu Prfsumme dazu

		rcall	uart_receive_byte_hex			; Record-Typ holen
		brtc	flash_programm_line_loop_3		; wenn T = 0 ok, weiter
		
		rjmp	flash_programm_end				; Fehler, Sprungweite zu gro fr direkten brts nach flash_receive_end
		
flash_programm_line_loop_3:
		add		TEMP_2,TEMP_0					; und zu Prfsumme dazu
		tst		TEMP_0							; Record-Typ Daten?							; 
		breq	flash_programm_byte_start		; ja, zum Einlesen
		
		cpi		TEMP_0,1						; End-Record?
		breq	flash_programm_line_loop_4
		
		rjmp	flash_programm_end				; nein, falsch, unbekannter Record

flash_programm_line_loop_4:
		rcall	uart_receive_byte_hex			; Prfsumme holen
		brtc	flash_programm_line_loop_end_record		; wenn T = 0 ok, weiter
		
		rjmp	flash_programm_end				; Fehler, Sprungweite zu gro fr direkten brts nach flash_receive_end
		
flash_programm_line_loop_end_record:
		add		TEMP_2,TEMP_0					; zu Prfsumme dazu
		brne	flash_programm_line_error		; nicht 0, also Prfsummenfehler
		
		sts		FLASH_BUF+1,ZH					; letzte Adresse +1 merken
		sts		FLASH_BUF,ZL

		rcall	flash_write_daten				; restliche Daten schreiben
						
		rjmp	flash_programm_end				; alles ok	

		; Y aktuelle Flash-Adresse
		; Z aktuelle Buffer-Adresse
		; TEMP_2 Prfsumme
		; TEMP_3 Anzahl Bytes in der folgenden Zeile

flash_programm_byte_start:						
		sbrc	FLAGS,FLAG_ADR					; Flash-Adresse schon gemerkt?
		rjmp	flash_programm_byte_loop_adr	; ja also neue Datenzeile, zum Adresstest

		sts		FLASH_ADR+1,YH					; Flash-Adresse merken
		sts		FLASH_ADR,YL
		sbr		FLAGS,(1<<FLAG_ADR)				; Flag Adresse gemerkt
		rjmp	flash_programm_byte_loop		; Einlesen starten

flash_programm_byte_loop_adr:	

		lds		TEMP_0,FLASH_ADR+1				; Adresse H-Byte holen
		andi	TEMP_0,high(FLASH_START)		; ausmaskieren auf 0x04
		mov		TEMP_1,YH
		andi	TEMP_1,high(FLASH_START)
		cp		TEMP_0,TEMP_1					; Adresse verndert von Control (0x00) auf Daten (0x40...0x7F)?
		brne	flash_programm_byte_write		; ja, also XOFF und erstmal schreiben

		cpi		YH,high(FLASH_START)			; 0x40, also Daten?
		brsh	flash_programm_byte_loop_adr_end; ja, dann Bufferende testen

		lds		TEMP_0,FLASH_ADR+1				; H-Byte holen
		cp		TEMP_0,YH						; andere Controlmap?
		brne	flash_programm_byte_write		; ja

		lds		TEMP_0,FLASH_ADR				; bei Control L-Byte holen
		andi	TEMP_0,0x80						; 128 Byte-Grenze der Control-Map
		mov		TEMP_1,YL
		andi	TEMP_1,0x80						; 128 Byte-Grenze der Control-Map
		cp		TEMP_0,TEMP_1					; andere Map?
		brne	flash_programm_byte_write		; ja, also XOFF und erstmal schreiben

flash_programm_byte_loop_adr_end:
		cpi		ZH,high(FLASH_END)				; Bufferende?
		brlo	flash_programm_byte_loop			; nein, nchster Test +16 Byte...

flash_programm_byte_write:				
		sts		FLASH_BUF+1,ZH					; letzte Adresse +1 merken
		sts		FLASH_BUF,ZL
		push	YH								; neue Flash-Adresse ist noch in Y, merken!!!
		push	YL
		rcall	flash_xon_xoff_write			; XOFF setzen und Bufferinhalt programmieren
		brts	flash_programm_line_error		; Fehler

		pop		YL								; neue Flash-Adresse zurck
		pop		YH
		sts		FLASH_ADR+1,YH					; und Flashadresse merken			
		sts		FLASH_ADR,YL

		; Z zeigt auf aktuelle Buffer-Adresse

flash_programm_byte_loop:
		rcall	uart_receive_byte_hex			; Datenbyte holen
		brts	flash_programm_end				; T gesetzt -> Abbruch wegen Fehler

		add		TEMP_2,TEMP_0					; zu Prfsumme dazu
		st		Z+,TEMP_0						; speichern
		dec		TEMP_3							; Anzahl Bytes in der Zeile
		brne	flash_programm_byte_loop

		rcall	uart_receive_byte_hex			; Prfsumme holen
		brts	flash_programm_end				; T gesetzt -> Abbruch wegen Fehler

		add		TEMP_2,TEMP_0					; zu Prfsumme dazu, sollte 0 ergeben
		brne	flash_programm_line_error		; falsch, Abbruch

		ldi		TEMP_0,'.'
		rcall	uart_send_byte

		rjmp	flash_programm_line_loop			; ok, nchste Zeile

flash_programm_line_error:
		load_p	Z,(2*text_flasherr_3)
		rcall	display_newline
		rcall	display_write_text

flash_programm_end:		
		rcall	uart_receive_byte_timeout		; brige CR/LF einsammeln
		brne	flash_programm_end		

		rcall	display_newline
		load_p	Z,(2*text_flash_prog_end)
		rcall	show_text_line

		load_p	Z,(2*text_taste)
		rcall	show_text_line

		rcall	uart_receive_byte				; auf Zeichen warten	

		clr		FLAGS

		rcall	flash_init						; Neu initialisieren wegen evtl. vernderter Maos
		rcall	display_init

		ret

;******************************* Sub-Flash *********************************
; Sub: ON XOFF und schreiben
; Parameter:    TEMP_3 Anzahl Bytes, Z Bufferadresse
; Return:       -
; Scratch-Reg:  -
;***************************************************************************

flash_xon_xoff_write:
		ldi		TEMP_0,X_OFF
		rcall	uart_send_byte					; XOFF senden
		
		load_p	Z,FLASH_END						; auf Ende setzen fr restliche Bytes
		push	TEMP_3							; Anzahl Bytes der Zeile

		rcall	uart_receive_byte_timeout		; Datenbyte holen mit Timeout

		mov		TEMP_1,ZL						; Anzahl empfangene Bytes seit XOFF		
		push	TEMP_1
		push	TEMP_2							; Prfsumme
		rcall	flash_write_daten
		pop		TEMP_2							; Prfsumme
		pop		TEMP_1							; Anzahl Timeout-Bytes
								
		load_p	Y,FLASH_END						; Zeiger auf Timeout-Buffer, Rohdaten
		load_p	Z,DDRAM_1						; Empfamgsbuffer Anfang
		
flash_xon_xoff_write_loop:
		ld		TEMP_0,Y+						; Byte holen
		rcall	check_nibble					; Konvert H-Nibble und Test
		brts	flash_xon_xoff_write_end		; Fehler
		
		mov		TEMP_3,TEMP_0
		swap	TEMP_3							; nach oben, Timeout beginnt immer mit erstem Datenbyte!
		
		dec		TEMP_1
		breq	flash_xon_xoff_write_get_nibble ; da fehlt ein Low-Nibble...		

		ld		TEMP_0,Y+						; Byte holen
		rcall	check_nibble					; Konvert L-Nibble und Test
		brts	flash_xon_xoff_write_end		; Fehler

		or		TEMP_0,TEMP_3					; Byte zusammenfgen
		st		Z+,TEMP_0						; und in Buffer
		add		TEMP_2,TEMP_0					; zur Prfsumme dazu
		pop		TEMP_3							; Anzahl Bytes der Zeile holen
		dec		TEMP_3
		push	TEMP_3							; und wieder sichern

		dec		TEMP_1
		brne	flash_xon_xoff_write_loop		; und weiter

		ldi		TEMP_0,X_ON
		rcall	uart_send_byte					; XON senden

		pop		TEMP_3							; Anzahl Bytes der Zeile zurck holen
		rjmp	flash_xon_xoff_write_end		; fertig

flash_xon_xoff_write_get_nibble:
		ldi		TEMP_0,X_ON
		rcall	uart_send_byte					; XON senden

		rcall	uart_receive_byte				; fehlendes Byte holen
		rcall	check_nibble
		brts	flash_xon_xoff_write_end

		or		TEMP_0,TEMP_3
		st		Z+,TEMP_0						; und speichern
		add		TEMP_2,TEMP_0					; zur Prfsumme dazu
		pop		TEMP_3							; Anzahl Bytes der Zeile holen
		dec		TEMP_3

	; Z zeigt auf die nchste Bufferadresse, Prfsumme in TEMP_2, restliche Anzahl in TEMP_3 
	
flash_xon_xoff_write_end:

		ret


