;************************** UART Routinen *************************

;******************************* Sub-Debug *********************************
; Sub: Sendet Text aus Programm-Flash ber die serielle Schnittstelle
; Parameter:	Z enthlt den Zeiger auf den Text im ROM
; Return:		-
; Scratch-Reg:	-
;***************************************************************************

uart_send_text:
		push	ZH
		push	ZL
		push	TEMP_0

uart_send_text_loop:
        lpm		TEMP_0,Z+
		tst		TEMP_0
		breq	uart_send_text_end					; Ende

        rcall   uart_send_byte

		rjmp    uart_send_text_loop					; weiter

uart_send_text_end:
		pop		TEMP_0
 		pop		ZL
		pop	ZH

        ret

;******************************* Sub-Debug *********************************
; Sub: Sendet NL/CR ber die serielle Schnittstelle
; Parameter:	-
; Return:		-
; Scratch-Reg:	-
;***************************************************************************

uart_send_newline:
        push    TEMP_0

		ldi     TEMP_0,$0A
        rcall   uart_send_byte
		ldi     TEMP_0,$0D
        rcall   uart_send_byte

        pop     TEMP_0

		ret

;******************************* Sub-UART **********************************
; Sub: Sendet ein Byte aus TEMP_0 als HEX-Dump ber UART
; Parameter: 	TEMP_0
; Return:      	-
; Scratch-Reg:	-
;***************************************************************************

uart_send_byte_hex:
        push TEMP_0
		push TEMP_1

        ldi  TEMP_1,7
        push TEMP_0
		swap TEMP_0
		andi TEMP_0,$0F
		ori  TEMP_0,$30								; nach Ziffer
		cpi  TEMP_0,$3A								; 0...9
    	brlo wb_1
        add  TEMP_0,TEMP_1                 			; A...F
wb_1:
        rcall uart_send_byte
		pop  TEMP_0
		andi TEMP_0,$0F
		ori  TEMP_0,$30                   			; nach Ziffer
		cpi  TEMP_0,$3A                   			; 0...9
		brlo wb_2
        add  TEMP_0,TEMP_1                			; A...F
wb_2:
        rcall uart_send_byte

        pop   TEMP_1
		pop   TEMP_0

        ret

;******************************* Sub-UART **********************************
; Sub: Sendet ein Byte aus TEMP_0 ber UART
; Parameter:	TEMP_0 Datenbyte
; Return:		-
; Scratch-Reg:	-
;***************************************************************************

uart_send_byte:
        sbis    UCSRA,UDRE                  		; Send-Register leer ?
        rjmp    uart_send_byte         	    		; nein

        out     UDR,TEMP_0

        ret

;******************************* Sub-UART **********************************
; Sub: Empfngt Byte ber UART
; Parameter:    -
; Return:       TEMP_0 Datenbyte
; Scratch-Reg:  -
;***************************************************************************

uart_receive_byte:	
		sbis	UCSRA,RXC				        ; warten auf Parameter-Byte
		rjmp 	uart_receive_byte				; nein

        in		TEMP_0,UDR						; holen

		ret

;******************************* Sub-UART **********************************
; Sub: Empfngt Bytes ber UART bis Timeout
; Parameter:    TEMP_1 * ca. 50ms
; Return:       TEMP_0 Datenbyte, Z = 0, bei Timeout Z = 1, TEMP_0 unverndert
; Scratch-Reg:  TEMP_1
;***************************************************************************

uart_receive_byte_timeout:
		push	YH
		push	YL
		
		clr		YH
		clr		YL
		ldi		TEMP_1,TIMEOUT					; fr ca. 0,5s Timeout

uart_receive_byte_timeout_loop:
		sbic	UCSRA,RXC				        ; #2 warten auf Byte
		rjmp 	uart_receive_byte_timeout_char	; Byte empfangen

		sbiw	Y,1								; #2	
		brne	uart_receive_byte_timeout_loop	; #2	6 Zyklen eine Schleife - 393216,
												; bei ca. 40% CPU-Last rund 600000 Zyklen, bei 12MHz ca. 50ms 

		dec		TEMP_1							; #1	
		brne	uart_receive_byte_timeout_loop	; #2

		rjmp	uart_receive_byte_timeout_end	; kam nichts mehr

uart_receive_byte_timeout_char:					; Byte da, Z = 0, Z = 1 Timeout
		in		TEMP_0,UDR						; abholen
		st		Z+,TEMP_0						; und als Roh-Daten speichern
		rjmp	uart_receive_byte_timeout_loop	; und weiter einlesen

uart_receive_byte_timeout_end:
		pop		YL
		pop		YH

		ret

;******************************* Sub-UART **********************************
; Sub: Empfngt 2 Byte als Hex ber UART und konvertiert nach binr
; Parameter:    -
; Return:       TEMP_0 Datenbyte
; Scratch-Reg:  -
;***************************************************************************

uart_receive_byte_hex:
		push	TEMP_1

		rcall	uart_receive_byte
		rcall	check_nibble
		brts	uart_receive_byte_hex_end		; Fehler	

		mov		TEMP_1,TEMP_0
		swap	TEMP_1							; oberes Nibble

		rcall	uart_receive_byte
		rcall	check_nibble
		brts	uart_receive_byte_hex_end		; Fehler	

		or		TEMP_0,TEMP_1					; unteres Nibble dazu nach TEMP_0

uart_receive_byte_hex_end:
		pop		TEMP_1

		ret
